<?php
namespace Teknisa\EST\Modules\Lancamentos\Controllers;

use Teknisa\EST\Services\ValidacoesEstoqueService;
use \Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DataSource\DataSet;

class Est11000ItemEntrada extends TeknisaCrud {

    protected $dataSourceName = "itlanctoest_entrada";
    protected $dataSourceNameToSave = "itlanctoest";
    //protected $parameterBagColumns = ['CDFILIMOVI','NRLANCESTQ'];

    protected $Est11000Service;
    protected $validacoesEstoqueService;

    protected $connection;

    public function __construct($crudService, $parameterBag, $Est11000Service, ValidacoesEstoqueService $validacoesEstoqueService, $entityManager) {
        parent::__construct($crudService,$parameterBag);

        $this->Est11000Service = $Est11000Service;
        $this->validacoesEstoqueService = $validacoesEstoqueService;

        $this->connection = $entityManager->getConnection();
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $this->connection->beginTransaction();
            $rows = $request->getDataset()->getRows();
            $row = $rows[0];
            $lancamentoRow = array_filter($rows, function($row){
                return isset($row['LANCAMENTO']);
            });
            
            $arrNewCodeFields = array('__using_unit', '__new_code_pk', '__table_pk', '__new_code_interval', '__new_code_length', '__new_code_column_size', '__cast_number', '__columns_to_return');
            $arrLocaisEst = array('CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE');
            $lancamentoRow = array_values($lancamentoRow)[0];
            $lancamento = $lancamentoRow['LANCAMENTO'];

            foreach($arrNewCodeFields as $field){
                $lancamento[$field] = $lancamentoRow[$field];
                unset($lancamentoRow[$field]);
            }
            
            $this->dataSourceNameToSave = 'lanctoestoq';
            $newRequest = new DTO\Request\Row ($lancamento, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
    	    parent::save($newRequest, $response);
    	    $columnsReturn = $response->getDataSets()[0]->getRows()[0];
    	    
    	    if(!(count($rows) == 1 && !isset($rows[0]['CDPRODMOVI']))){ //confere se tem alterações em items para salvar
        	    foreach($rows as &$row){
        	        $row['NRLANCESTQ'] = $columnsReturn['NRLANCESTQ'];
        	        $row['DTLANCMOVI'] = substr($row['DTLANCMOVI'],0, 10);
        	        $row['__columns_to_return'] = array('NRLANCESTQ');
        	        
        	        foreach($arrLocaisEst as $localField){
        	            if(empty($row[$localField]))
        	                $row[$localField] = ' ';
        	        }
        	    }
        	    
        	    $dataSetProdutos = new DataSet('itlanctoest', $rows);
        	    $requestProdutos = new DTO\Request\DataSet($dataSetProdutos, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                
                $filial = $row['CDFILIMOVI'];
                $dtLancamento = $row['DTLANCMOVI'];
                $operacao = 'E';
                $sistema = 'EST';
                $dtEmissao = $row['DTLANCMOVI']; //verificar depois
                
                $this->Est11000Service->verificaInventarioRotativo($lancamento['CDFILIMOVI'],
                                                              $lancamento['DTLANCESTQ']);
            
                $this->validacoesEstoqueService->validaDataLancamento($filial, $dtLancamento, $operacao, $sistema, $dtEmissao);
                
                $this->dataSourceNameToSave = 'itlanctoest';
                parent::save($requestProdutos,$response);
                
                foreach($rows as $row){
                    $this->validacoesEstoqueService->atualizaProdlcestoq($row['CDFILIAL'], $row['CDALMOXARIFE'], $row['CDLOCALESTOQ'] , $row['CDPRODUTO']);
                }
                
                $this->validacoesEstoqueService->executaProcedureCustoLiquido($row["CDFILIAL"], $row["NRLANCESTQ"]);
                $this->validacoesEstoqueService->atualizaEstoque($row["CDFILIAL"], $row["NRLANCESTQ"]);
    	    }
            
            $this->connection->commit();
            
        } catch (\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $item = $request->getRow();
        $this->connection->beginTransaction();
        try {
            $this->Est11000Service->verificaInventarioRotativo($item['CDFILIMOVI'],
                                                              substr($item['DTLANCMOVI'], 0, 10));

            $this->Est11000Service->processDeleteItem($item);

            $this->validacoesEstoqueService->atualizaEstoque($item["CDFILIAL"], $item["NRLANCESTQ"]);

            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}